package org.hathitrust.extractedfeatures;

import org.apache.spark.api.java.function.VoidFunction;
import org.apache.spark.util.DoubleAccumulator;
import org.json.JSONArray;
import org.json.JSONObject;

/*
class PagedJSON implements Function<String, Boolean> {

	private static final long serialVersionUID = 1L;

	public Boolean call(String s) { return s.contains("a"); }
}
 */


public class PerVolumeJSON implements VoidFunction<String> 
{
	private static final long serialVersionUID = 1L;
	protected String _input_dir;

	protected WhitelistBloomFilter _whitelist_bloomfilter;

	protected String _solr_url;
	protected String _output_dir;
	protected int    _verbosity;
	
	protected DoubleAccumulator _progress_accum;
	protected double            _progress_step;
	
	public PerVolumeJSON(String input_dir, String whitelist_filename,
				         String solr_url, String output_dir, int verbosity, 
					     DoubleAccumulator progress_accum, double progress_step)
	{
		_input_dir  = input_dir;
		
		if (whitelist_filename != null) {
			_whitelist_bloomfilter = new WhitelistBloomFilter(whitelist_filename,true);
		}
		
		_solr_url   = solr_url;
		_output_dir = output_dir;
		_verbosity  = verbosity;
		
		_progress_accum = progress_accum;
		_progress_step  = progress_step;
	}
	
	//public Iterator<String> call(String json_file_in) 
	public void call(String json_file_in) 
	{ 
		JSONObject extracted_feature_record = JSONClusterFileIO.readJSONFile(_input_dir + "/" + json_file_in);
		
		String volume_id = extracted_feature_record.getString("id");
		
		//JSONObject ef_metadata = extracted_feature_record.getJSONObject("metadata");
		//String title= ef_metadata.getString("title");
		
		JSONObject ef_features = extracted_feature_record.getJSONObject("features");
		
		
		int ef_page_count = ef_features.getInt("pageCount");
		
		if (_verbosity >= 1) {
			System.out.println("Processing: " + json_file_in);
			System.out.println("  pageCount = " + ef_page_count);
		}
		
		JSONArray ef_pages = ef_features.getJSONArray("pages");
		int ef_num_pages = ef_pages.length();
		
		// Make directory for page-level JSON output
		String json_dir = ClusterFileIO.removeSuffix(json_file_in,".json.bz2");
		String page_json_dir = json_dir + "/pages";
		
		if (_output_dir != null) {
			ClusterFileIO.createDirectoryAll(_output_dir + "/" + page_json_dir);
		}
		
		//ArrayList<String> ids = new ArrayList<String>(ef_num_pages);
		for (int i = 0; i < ef_page_count; i++) {
			String formatted_i = String.format("page-%06d", i);
			String page_id = volume_id + "." + formatted_i;
		
			if (_verbosity >= 2) {
			  System.out.println("  Page: " + page_id);
			}
			
			String output_json_bz2 = page_json_dir +"/" + formatted_i + ".json.bz2";
			//ids.add(output_json_bz2); // ****
			
			if (i==0) {
				System.out.println("Sample output JSON page file: " + output_json_bz2);
			}
			
			JSONObject ef_page = ef_pages.getJSONObject(i);

			if (ef_page != null) {
				// Convert to Solr add form
				JSONObject solr_add_doc_json 
					= SolrDocJSON.generateSolrDocJSON(volume_id, page_id, ef_page, _whitelist_bloomfilter);

			
				if ((_verbosity >=2) && (i==20)) {
					System.out.println("==================");
					System.out.println("Sample output Solr add JSON [page 20]: " + solr_add_doc_json.toString());
					System.out.println("==================");
				}
				
							
				if (_solr_url != null) {
					if ((_verbosity >=2) && (i==20)) {
						System.out.println("==================");
						System.out.println("Posting to: " + _solr_url);
						System.out.println("==================");
					}
					SolrDocJSON.postSolrDoc(_solr_url, solr_add_doc_json);
				}

				if (_output_dir != null) {
					if ((_verbosity >=2) && (i==20)) {
						System.out.println("==================");
						System.out.println("Saving to: " + _output_dir);
						System.out.println("==================");
					}
					SolrDocJSON.saveSolrDoc(solr_add_doc_json, _output_dir + "/" + output_json_bz2);
				}
			}
			else {
				System.err.println("Skipping: " + page_id);
			}
			
		}
		
		
		//ids.add(volume_id);
		_progress_accum.add(_progress_step);
		
		//return ids.iterator();
	}
}

